//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
#include "FormSettings.h"
#include "FormAccount.h"
#include "LogUnit.h"
#include "Log.h"
#include "UaMain.h"
#include "ControlQueue.h"
#include "Callback.h"
#include "CallbackQueue.h"
#include <assert.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
	lbl2ndParty->Caption = "";
	lblCallState->Caption = "";
	incoming_call = false;
	connected = false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.Read(asConfigFile);
	if (this->BorderStyle != bsSingle)
	{
		this->Width = appSettings.frmMain.iWidth;
		this->Height = appSettings.frmMain.iHeight;
	}
	this->Top = appSettings.frmMain.iPosY;
	this->Left = appSettings.frmMain.iPosX;
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.frmMain.bWindowMaximized)
		this->WindowState = wsMaximized;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	UpdateCallHistory();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.frmMain.bWindowMaximized = (this->WindowState == wsMaximized);
	if (!appSettings.frmMain.bWindowMaximized)
	{
		// these values are meaningless is wnd is maximized
		appSettings.frmMain.iWidth = this->Width;
		appSettings.frmMain.iHeight = this->Height;
		appSettings.frmMain.iPosY = this->Top;
		appSettings.frmMain.iPosX = this->Left;
	}
	appSettings.Write(asConfigFile);

	CanClose = true;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::actShowAboutExecute(TObject *Sender)
{
	frmAbout->ShowModal();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowSettingsExecute(TObject *Sender)
{
    UaConf tmp = appSettings.uaConf;
	frmSettings->ShowModal();
	if (appSettings.uaConf != tmp)
	{
        StatusBar->SimpleText = "Restarting UA...";
		Ua::Instance().Restart();
	}
	if (appSettings.uaConf.logMessages != tmp.logMessages)
	{
		UA->SetMsgLogging(appSettings.uaConf.logMessages);
	}
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);
	if (frmLog->Visible)
	{
    	frmLog->UpdateUi();
	}

	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.Write(asConfigFile);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormShow(TObject *Sender)
{
    static bool once = false;
    if (!once)
    {
		once = true;
		CLog::Instance()->SetLevel(E_LOG_TRACE);
		CLog::Instance()->callbackLog = frmLog->OnLog;
#if 0
		frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);
		frmLog->BorderStyle = bsNone;
		frmLog->Align = alClient;
		frmLog->Parent = tsLog;
		frmLog->Visible = true;
#endif
	}
	LOG("\n===================\nApplication started\n");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
	UA->Destroy();
	UA_CB->Destroy();
	CLog::Instance()->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowLogExecute(TObject *Sender)
{
	if (!frmLog->Visible)
		frmLog->Show();
	frmLog->BringToFront();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::tmrStartupTimer(TObject *Sender)
{
	tmrStartup->Enabled = false;

	Ua::Instance().Start();
}
//---------------------------------------------------------------------------

void TfrmMain::UpdateCallHistory(void)
{
	cbCallURI->Clear();
	for (unsigned int i=0; i<appSettings.uaConf.callHistory.size(); i++)
	{
		UaConf::HistoryEntry &entry = appSettings.uaConf.callHistory[i];
		cbCallURI->Items->Add(entry.uri.c_str());
	}
}

void __fastcall TfrmMain::btnMakeCallClick(TObject *Sender)
{
	if (incoming_call == false)
	{
		AnsiString target = cbCallURI->Text.Trim();
		if (target != "")
		{
			UA->Call(0, target);
			UaConf::HistoryEntry entry;
			entry.uri = target.c_str();
			appSettings.uaConf.AddCallHistoryEntry(entry);
			UpdateCallHistory();
		}
	}
	else
	{
    	UA->Answer(0);
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnHangupClick(TObject *Sender)
{
	UA->Hangup(0);
	lbl2ndParty->Caption = "";
	lblCallState->Caption = "";
	incoming_call = false;
	connected = false;
}
//---------------------------------------------------------------------------

AnsiString TfrmMain::CleanUri(AnsiString uri)
{
	AnsiString res = uri;
	int start = uri.Pos("<");
	if (start == 0)
		start = 1;
	int end = uri.Pos(">");
	if (end == 0 || (uri.Pos(";") < end))
		end = uri.Pos(";");
	if (start && end && end > start)
	{
		res = uri.SubString(start, end-start);
	}
	return res;
}

void __fastcall TfrmMain::tmrCallbackPollTimer(TObject *Sender)
{
	Callback cb;
	if (UA_CB->GetCallback(cb) != 0)
	{
    	return;
	}
	switch (cb.type)
	{
		case Callback::CALL_STATE:
		{
			AnsiString asStateText;
			switch(cb.state)
			{
			case Callback::CALL_STATE_INCOMING:
				asStateText = "Incoming call";
				incoming_call = true;
				break;
			case Callback::CALL_STATE_RINGING:
				asStateText = "Ringback";
				break;
			case Callback::CALL_STATE_PROGRESS:
				asStateText = "Call state progress";
				break;
			case Callback::CALL_STATE_ESTABLISHED:
				asStateText = "Connected";
				connected = true;
				break;
			case Callback::CALL_STATE_CLOSED:
				asStateText = "End of call";
				incoming_call = false;
				connected = false;
				break;
			default:
				assert(!"Unhandled call state");
				break;
			}
			lbl2ndParty->Caption = CleanUri(cb.caller);
			lblCallState->Caption = asStateText;
			break;
		}
		case Callback::REG_STATE:
		{
			AnsiString asRegText;
			switch(cb.reg_state)
			{
			case Callback::REG_STATE_REGISTERING:
				asRegText = "Registering";
				break;
			case Callback::REG_STATE_REGISTER_OK:
				asRegText = "Registered";
				break;
			case Callback::REG_STATE_REGISTER_FAIL:
				asRegText = "Error registering";
				break;
			case Callback::REG_STATE_UNREGISTERING:
				asRegText = "Unregistering";
				break;
			case Callback::REG_STATE_UNREGISTER_OK:
				asRegText = "Unregistered";
				break;
			case Callback::REG_STATE_UNREGISTER_FAIL:
				asRegText = "Unregistering error";
				break;
			default:
				assert(!"Unhandled reg state");
				break;
			}
			if (appSettings.uaConf.accounts.size())
			{
				asRegText.cat_sprintf(" %s@%s",
					appSettings.uaConf.accounts[0].user.c_str(),
					appSettings.uaConf.accounts[0].reg_server.c_str());
			}
			StatusBar->SimpleText = asRegText;
			break;
		}
		case Callback::APP_STATE:
		{
			switch(cb.app_state)
			{
			case Callback::APP_START_FAILED:
				StatusBar->SimpleText = "Failed to start application";
				break;
			default:
				assert(!"Unhandled app state");
				break;
			}
			break;
		}
		default:
		{
			assert(!"Unhandled callback type");
		}
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnDialClick(TObject *Sender)
{
	char digit = '\0';
	TButton* btn = dynamic_cast<TButton*>(Sender);
	assert(btn);
	switch(btn->Tag)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
		digit = '0' + btn->Tag;
		break;
	case 10:
		digit = '*';
		break;
	case 11:
		digit = '#';
		break;
	default:
		assert(!"Unhandled button tag value");
	}

	if (connected)
	{
    	UA->SendDigit(0, digit);
	}
	else
	{
		cbCallURI->Text = cbCallURI->Text + digit;
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnBackspaceClick(TObject *Sender)
{
	cbCallURI->Text = cbCallURI->Text.SubString(1, cbCallURI->Text.Length() - 1);
	cbCallURI->SelStart = cbCallURI->Text.Length();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormClose(TObject *Sender, TCloseAction &Action)
{
	Ua::Instance().Quit();
}
//---------------------------------------------------------------------------

